@@ -11,22 +11,29 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef __GTKSTYLE_H__
 #define __GTKSTYLE_H__
 
+#if defined(LIBXFCEGUI4_COMPILATION) || !defined(XFCE_DISABLE_DEPRECATED)
+
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <pango/pango-font.h>
 
-gchar *get_style (GtkWidget *, gchar *, gchar *);
-GdkGC *get_style_gc (GtkWidget *, gchar *, gchar *);
-PangoFontDescription *get_font_desc (GtkWidget *);
-PangoContext *pango_get_context (GtkWidget *);
+G_BEGIN_DECLS
+
+gchar *get_style (GtkWidget *widget, const gchar *name, const gchar *state);
+GdkGC *get_style_gc (GtkWidget *widget, const gchar *state, const gchar *style);
+PangoFontDescription *get_font_desc (GtkWidget *widget);
+PangoContext *pango_get_context (GtkWidget *widget);
+
+G_END_DECLS
+
+#endif /* !XFCE_DISABLE_DEPRECATED */
 
 #endif
