@@ -1,5 +1,5 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #include <libxfcegui4/libxfcegui4.h>
 
@@ -20,13 +20,13 @@ netk_window_state_get_type (void)
       { NETK_WINDOW_STATE_HIDDEN, "NETK_WINDOW_STATE_HIDDEN", "hidden" },
       { NETK_WINDOW_STATE_FULLSCREEN, "NETK_WINDOW_STATE_FULLSCREEN", "fullscreen" },
       { NETK_WINDOW_STATE_URGENT, "NETK_WINDOW_STATE_URGENT", "urgent" },
+      { NETK_WINDOW_STATE_DEMANDS_ATTENTION, "NETK_WINDOW_STATE_DEMANDS_ATTENTION", "demands-attention" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("NetkWindowState", values);
   }
   return etype;
 }
-
 GType
 netk_window_actions_get_type (void)
 {
@@ -55,7 +55,6 @@ netk_window_actions_get_type (void)
   }
   return etype;
 }
-
 GType
 netk_window_type_get_type (void)
 {
@@ -78,7 +77,6 @@ netk_window_type_get_type (void)
   return etype;
 }
 
-
 /* enumerations from "netk-pager.h" */
 GType
 netk_pager_display_mode_get_type (void)
@@ -95,7 +93,6 @@ netk_pager_display_mode_get_type (void)
   return etype;
 }
 
-
 /* enumerations from "netk-tasklist.h" */
 GType
 netk_tasklist_grouping_type_get_type (void)
@@ -114,5 +111,4 @@ netk_tasklist_grouping_type_get_type (void)
 }
 
 
-/* Generated data ends here */
 
