@@ -15,9 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -62,8 +61,6 @@ enum {
     PROP_USE_UNDERLINE,
 };
 
-static void xfce_app_menu_item_class_init(XfceAppMenuItemClass *klass);
-static void xfce_app_menu_item_init(XfceAppMenuItem *app_menu_item);
 static void xfce_app_menu_item_set_property(GObject *object,
                                             guint prop_id,
                                             const GValue *value,
@@ -72,15 +69,15 @@ static void xfce_app_menu_item_get_property(GObject *object,
                                             guint prop_id,
                                             GValue *value,
                                             GParamSpec *pspec);
+static void xfce_app_menu_item_size_request(GtkWidget *widget,
+                                            GtkRequisition *req);
 static void xfce_app_menu_item_finalize(GObject *object);
 
-static void xfce_app_menu_item_realize(GtkWidget *widget);
-
-static void xfce_app_menu_item_update_icon(XfceAppMenuItem *app_menu_item);
-
 static void _command_activate_cb(GtkMenuItem *menu_item,
                                  gpointer user_data);
 
+static gint _xfce_app_menu_item_icon_size = -1;
+
 
 G_DEFINE_TYPE(XfceAppMenuItem, xfce_app_menu_item, GTK_TYPE_IMAGE_MENU_ITEM)
 
@@ -222,7 +219,7 @@ xfce_app_menu_item_class_init(XfceAppMenuItemClass *klass)
     
     g_type_class_add_private(klass, sizeof(XfceAppMenuItemPriv));
     
-    widget_class->realize = xfce_app_menu_item_realize;
+    widget_class->size_request = xfce_app_menu_item_size_request;
     
     gobject_class->finalize = xfce_app_menu_item_finalize;
     gobject_class->set_property = xfce_app_menu_item_set_property;
@@ -257,6 +254,11 @@ xfce_app_menu_item_class_init(XfceAppMenuItemClass *klass)
             g_param_spec_boolean("use-underline", _("Use underline"),
                                  _("Whether or not to use an underscore in the label as a keyboard mnemonic"),
                                  FALSE, G_PARAM_READABLE | G_PARAM_WRITABLE));
+    
+    if(-1 == _xfce_app_menu_item_icon_size) {
+        gtk_icon_size_lookup(GTK_ICON_SIZE_MENU,
+                             &_xfce_app_menu_item_icon_size, NULL);
+    }
 }
 
 static void
@@ -355,6 +357,43 @@ xfce_app_menu_item_get_property(GObject *object, guint prop_id, GValue *value,
     }
 }
 
+static void
+xfce_app_menu_item_size_request(GtkWidget *widget,
+                                GtkRequisition *req)
+{
+
+    XfceAppMenuItem *app_menu_item = XFCE_APP_MENU_ITEM(widget);
+    GtkWidget *img;
+    GdkPixbuf *pix;
+    
+    if(app_menu_item->priv->icon_path && !app_menu_item->priv->icon_set) {
+        img = gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(app_menu_item));
+        if(!img) {
+            img = gtk_image_new();
+            gtk_widget_show(img);
+            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(app_menu_item), img);
+        }
+        
+        pix = gdk_pixbuf_new_from_file_at_size(app_menu_item->priv->icon_path,
+                                               _xfce_app_menu_item_icon_size,
+                                               _xfce_app_menu_item_icon_size,
+                                               NULL);
+        
+        if(pix) {
+            gtk_image_set_from_pixbuf(GTK_IMAGE(img), pix);
+            g_object_unref(G_OBJECT(pix));
+            app_menu_item->priv->icon_set = TRUE;
+        } else {
+            g_free(app_menu_item->priv->icon_path);
+            app_menu_item->priv->icon_path = NULL;
+        }
+    }
+    
+    /* let the parent do its magic */
+    GTK_WIDGET_CLASS(xfce_app_menu_item_parent_class)->size_request(widget,
+                                                                    req);
+}
+
 static void
 xfce_app_menu_item_finalize(GObject *object)
 {
@@ -372,54 +411,6 @@ xfce_app_menu_item_finalize(GObject *object)
     G_OBJECT_CLASS(xfce_app_menu_item_parent_class)->finalize(object);
 }
 
-static void
-xfce_app_menu_item_realize(GtkWidget *widget)
-{
-    XfceAppMenuItem *app_menu_item = XFCE_APP_MENU_ITEM(widget);
-
-    GTK_WIDGET_CLASS(xfce_app_menu_item_parent_class)->realize(widget);
-
-    xfce_app_menu_item_update_icon(app_menu_item);
-}
-
-static void
-xfce_app_menu_item_update_icon(XfceAppMenuItem *app_menu_item)
-{
-    GtkWidget *img;
-
-    if(!GTK_WIDGET_REALIZED(app_menu_item))
-        return;
-
-    img = gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(app_menu_item));
-    if(!img) {
-        img = gtk_image_new();
-        gtk_widget_show(img);
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(app_menu_item), img);
-    } else
-        gtk_image_clear(GTK_IMAGE(img));
-
-    if(app_menu_item->priv->icon_name) {
-        GtkIconTheme *itheme = gtk_icon_theme_get_default();
-
-        if(gtk_icon_theme_has_icon(itheme, app_menu_item->priv->icon_name)) {
-            gtk_image_set_from_icon_name(GTK_IMAGE(img), app_menu_item->priv->icon_name,
-                                         GTK_ICON_SIZE_MENU);
-        }
-    } else if(app_menu_item->priv->icon_path) {
-        GdkPixbuf *pix;
-        gint w, h;
-
-        gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &w, &h);
-        
-        pix = gdk_pixbuf_new_from_file_at_scale(app_menu_item->priv->icon_path,
-                                                w, h, TRUE, NULL);
-        if(pix) {
-            gtk_image_set_from_pixbuf(GTK_IMAGE(img), pix);
-            g_object_unref(G_OBJECT(pix));
-        }
-    }
-}
-
 /**
  * xfce_app_menu_item_new:
  * @returns: A new #XfceAppMenuItem.
@@ -429,7 +420,7 @@ xfce_app_menu_item_update_icon(XfceAppMenuItem *app_menu_item)
  * Since 4.1
  **/
 GtkWidget *
-xfce_app_menu_item_new()
+xfce_app_menu_item_new(void)
 {
     return g_object_new(XFCE_TYPE_APP_MENU_ITEM, NULL);
 }
@@ -692,48 +683,25 @@ void
 xfce_app_menu_item_set_icon_name(XfceAppMenuItem *app_menu_item,
         const gchar *filename)
 {
+    GtkWidget *img;
+    
     g_return_if_fail(XFCE_IS_APP_MENU_ITEM(app_menu_item));
 
-    g_free(app_menu_item->priv->icon_name);
-    app_menu_item->priv->icon_name = NULL;
-    g_free(app_menu_item->priv->icon_path);
-    app_menu_item->priv->icon_path = NULL;
+    if(app_menu_item->priv->icon_name)
+        g_free(app_menu_item->priv->icon_name);
 
-    if(filename) {
-        if(g_path_is_absolute(filename))
-            app_menu_item->priv->icon_path = g_strdup(filename);
-        else {
-            gchar *p, *q;
-            gsize len;
-
-            /* yes, there are really broken .desktop files out there
-             * messed up like this */
-
-            /* first make sure we aren't a weird relative path */
-            p = g_strrstr(filename, G_DIR_SEPARATOR_S);
-            if(p)
-                p++;
-            else
-                p = (gchar *)filename;
-
-            len = strlen(p);
-
-            /* now make sure we don't have an extension */
-            q = g_strrstr(p, ".");
-            if(q && (!strcmp(q, ".png") || !strcmp(q, ".svg")
-                     || !strcmp(q, ".jpg") || !strcmp(q, ".gif")
-                     || !strcmp(q, ".bmp")))
-            {
-                len -= strlen(q);
-            }
-
-            /* whatever's left... */
-            if(p[0] && len)
-                app_menu_item->priv->icon_name = g_strndup(p, len);
-        }
+    if(!filename) {
+        app_menu_item->priv->icon_name = NULL;
+        return;
     }
 
-    xfce_app_menu_item_update_icon(app_menu_item);
+    app_menu_item->priv->icon_name = g_strdup(filename);
+    app_menu_item->priv->icon_path = xfce_themed_icon_lookup(filename,
+                                                _xfce_app_menu_item_icon_size);
+    
+    img = gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(app_menu_item));
+    if(img)
+        gtk_image_set_from_pixbuf(GTK_IMAGE(img), NULL);
 }
 
 /**
@@ -826,10 +794,7 @@ xfce_app_menu_item_get_icon_name(XfceAppMenuItem *app_menu_item)
 {
     g_return_val_if_fail(XFCE_IS_APP_MENU_ITEM(app_menu_item), NULL);
     
-    if(app_menu_item->priv->icon_name)
-        return app_menu_item->priv->icon_name;
-    else
-        return app_menu_item->priv->icon_path;
+    return app_menu_item->priv->icon_name;
 }
 
 /**
